(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var HTTP = Package.http.HTTP;
var HTTPInternals = Package.http.HTTPInternals;

/* Package-scope variables */
var SandstormAutoupdateApps;

(function(){

//////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                  //
// packages/sandstorm-autoupdate-apps/autoupdate-apps.js                                            //
//                                                                                                  //
//////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                    //
// Sandstorm - Personal Cloud Sandbox                                                               // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                            // 2
// All rights reserved.                                                                             // 3
//                                                                                                  // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                  // 5
// you may not use this file except in compliance with the License.                                 // 6
// You may obtain a copy of the License at                                                          // 7
//                                                                                                  // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                     // 9
//                                                                                                  // 10
// Unless required by applicable law or agreed to in writing, software                              // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                         // 13
// See the License for the specific language governing permissions and                              // 14
// limitations under the License.                                                                   // 15
                                                                                                    // 16
SandstormAutoupdateApps = {};                                                                       // 17
                                                                                                    // 18
SandstormAutoupdateApps.updateAppIndex = function (db) {                                            // 19
  var appUpdatesEnabledSetting = db.collections.settings.findOne({_id: "appUpdatesEnabled"});       // 20
  var appUpdatesEnabled = appUpdatesEnabledSetting && appUpdatesEnabledSetting.value;               // 21
  if (!appUpdatesEnabled) {                                                                         // 22
    // It's much simpler to check appUpdatesEnabled here rather than reactively deactivate the      // 23
    // timer that triggers this call.                                                               // 24
    return;                                                                                         // 25
  }                                                                                                 // 26
  var appIndexUrl = db.collections.settings.findOne({_id: "appIndexUrl"}).value;                    // 27
  var appIndex = db.collections.appIndex;                                                           // 28
  var data = HTTP.get(appIndexUrl + "/apps/index.json").data;                                       // 29
  data.apps.forEach(function (app) {                                                                // 30
    app._id = app.appId;                                                                            // 31
                                                                                                    // 32
    var oldApp = appIndex.findOne({_id: app.appId});                                                // 33
    app.hasSentNotifications = false;                                                               // 34
    appIndex.upsert({_id: app._id}, app);                                                           // 35
    if ((!oldApp || app.versionNumber > oldApp.versionNumber) &&                                    // 36
        db.collections.userActions.findOne({appId: app.appId})) {                                   // 37
      var pack = db.collections.packages.findOne({_id: app.packageId});                             // 38
      var url = appIndexUrl + "/packages/" + app.packageId;                                         // 39
      if (pack) {                                                                                   // 40
        if (pack.status === "ready") {                                                              // 41
          if (pack.appId && pack.appId !== app.appId) {                                             // 42
            console.error("app index returned app ID and package ID that don't match:",             // 43
                          JSON.stringify(app));                                                     // 44
          } else {                                                                                  // 45
            db.sendAppUpdateNotifications(app.appId, app.packageId, app.name, app.versionNumber,    // 46
              app.version);                                                                         // 47
          }                                                                                         // 48
        } else {                                                                                    // 49
          var newPack = Packages.findAndModify({                                                    // 50
            query: {_id: app.packageId},                                                            // 51
            update: {$set: {isAutoUpdated: true}},                                                  // 52
          });                                                                                       // 53
          if (newPack.status === "ready") {                                                         // 54
            // The package was marked as ready before we applied isAutoUpdated=true. We should send
            // notifications ourselves to be sure there's no timing issue (sending more than one is
            // fine, since it will de-dupe).                                                        // 57
            if (pack.appId && pack.appId !== app.appId) {                                           // 58
              console.error("app index returned app ID and package ID that don't match:",           // 59
                            JSON.stringify(app));                                                   // 60
            } else {                                                                                // 61
              db.sendAppUpdateNotifications(app.appId, app.packageId, app.name, app.versionNumber,  // 62
                app.version);                                                                       // 63
            }                                                                                       // 64
          } else if (newPack.status === "failed") {                                                 // 65
            // If the package has failed, retry it                                                  // 66
            db.startInstall(app.packageId, url, true, true);                                        // 67
          }                                                                                         // 68
        }                                                                                           // 69
      } else {                                                                                      // 70
        db.startInstall(app.packageId, url, false, true);                                           // 71
      }                                                                                             // 72
    }                                                                                               // 73
  });                                                                                               // 74
};                                                                                                  // 75
                                                                                                    // 76
Meteor.methods({                                                                                    // 77
  updateApps: function(appUpdates) {                                                                // 78
    var db = this.connection.sandstormDb;                                                           // 79
    var backend = this.connection.sandstormBackend;                                                 // 80
                                                                                                    // 81
    _.forEach(appUpdates, function (val, appId) {                                                   // 82
      var pack = db.collections.packages.findOne({_id: val.packageId, appId: appId});               // 83
      if (!pack || !pack.manifest) {                                                                // 84
        console.error("Newer app not installed", val.name);                                         // 85
      } else {                                                                                      // 86
        db.addUserActions(val.packageId);                                                           // 87
        db.upgradeGrains(appId, val.version, val.packageId, backend);                               // 88
        Meteor.call("deleteUnusedPackages", appId);                                                 // 89
      }                                                                                             // 90
    });                                                                                             // 91
  },                                                                                                // 92
});                                                                                                 // 93
                                                                                                    // 94
//////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-autoupdate-apps'] = {
  SandstormAutoupdateApps: SandstormAutoupdateApps
};

})();

//# sourceMappingURL=sandstorm-autoupdate-apps.js.map
